/*H************************************************************************\
 $Workfile: Acs.h $
 
  Notes: 
 
 --------------------------------------------------------------------------
 Copyright Visual Systems Inc., All Rights Reserved.
 --------------------------------------------------------------------------
 
 See bottom of file for revision history.
 
\************************************************************************H*/

#ifndef __ACS_H
#define __ACS_H

#pragma pack(push, __ACS__PACKING__)
#pragma pack(1)

#include "CygNetDefs.h"

/* ACS Version */
#define SECSERV_VERSION          1


/* ACS Error Codes (base 0) */
#define SECSERV_NO_ERROR         (ACS_ERROR_BASE + 0)
#define SECSERV_KEY_NOT_FOUND    (ACS_ERROR_BASE + 2)
#define SECSERV_DUPLICATE_KEY    (ACS_ERROR_BASE + 3)
#define SECSERV_ACCESS_DENIED    (ACS_ERROR_BASE + 4)
#define SECSERV_INVALID_KEY      (ACS_ERROR_BASE + 5)
#define SECSERV_SEMAPHORE_LOCK   (ACS_ERROR_BASE + 15)


/* ACS Key Structure */
#define NUM_SECSERV_KEYS         18
#define SECSERV_KEY_LENGTH       100
typedef char SECKey[SECSERV_KEY_LENGTH];

#define ACS_USERID_SIZE          64

/* Message Header */
struct ACS_MESSAGE_HEADER
{
    unsigned short type;
    unsigned short count;
    unsigned short error;
    unsigned short version;
};

/* Security Levels */
#define USER_SECURITY        '0'


/* Access Levels */
#define NO_ACCESS            'N'
#define READ_ACCESS          'R'
#define UPDATE_ACCESS        'U'
#define ADD_ACCESS           'A'
#define DELETE_ACCESS        'D'
#define ADMIN_ACCESS         'M'

/* Access Levels, enumerated*/
enum EAcsAccessLevel
{
    eACS_ACCESS_NONE,
    eACS_ACCESS_READ,
    eACS_ACCESS_UPDATE,
    eACS_ACCESS_ADD,
    eACS_ACCESS_DELETE,
    eACS_ACCESS_ADMIN,
    eACS_ACCESS_COUNT
};

/* Security Key Array Offsets */
#define KEY_TYPE_OFFSET      0
#define KEY_ID_OFFSET        1
#define KEY_LOC_OFFSET       2
#define APP_OFFSET           3
#define EVENT_OFFSET         11
#define SUB_OFFSET           19
#define LIST_TYPE_OFFSET     27
#define SEC_INFO_OFFSET      28
#define AUTH_LEVEL_OFFSET    29
#define SEC_ID_OFFSET        30
#define PERMISSION_OFFSET    94


// Request Types (base 300)
//
#define SECSERV_GET_VERSION  0
#define USER_OK              (ACS_MESSAGE_BASE + 0)
#define ADD_KEY              (ACS_MESSAGE_BASE + 1)
#define DELETE_KEY           (ACS_MESSAGE_BASE + 2)
#define UPDATE_KEY           (ACS_MESSAGE_BASE + 3)
#define READ_KEY             (ACS_MESSAGE_BASE + 4)
#define GET_USER_ACCESS      (ACS_MESSAGE_BASE + 5)
#define GET_ACS_INFO         (ACS_MESSAGE_BASE + 6)

// Retired request types, still processed
//
#define USER_OK_OLD          1
#define ADD_KEY_OLD          3
#define DELETE_KEY_OLD       4
#define UPDATE_KEY_OLD       5
#define READ_KEY_OLD         6


/* Request and response blocks */
struct USER_OK_REQ
{
    unsigned short type;
    unsigned short count;
    unsigned short error;
    unsigned short version;
    char           user[ACS_USERID_SIZE];
    char           app[8];
    char           event[8];
    char           event_type;
    char           access;
};

struct USER_OK_RESP
{
    unsigned short type;
    unsigned short count;
    unsigned short error;
    unsigned short version;
    char           ok;
};


struct ADD_KEY_REQ
{
    unsigned short type;
    unsigned short count;
    unsigned short error;
    unsigned short version;
    char           user[ACS_USERID_SIZE];
    SECKey         key;
};

struct ADD_KEY_RESP
{
    unsigned short type;
    unsigned short count;
    unsigned short error;
    unsigned short version;
};


struct DELETE_KEY_REQ
{
    unsigned short type;
    unsigned short count;
    unsigned short error;
    unsigned short version;
    char           user[ACS_USERID_SIZE];
    SECKey         key;
};

struct DELETE_KEY_RESP
{
    unsigned short type;
    unsigned short count;
    unsigned short error;
    unsigned short version;
};


struct UPDATE_KEY_REQ
{
    unsigned short type;
    unsigned short count;
    unsigned short error;
    unsigned short version;
    char           user[ACS_USERID_SIZE];
    SECKey         old_key;
    SECKey         new_key;
};

struct UPDATE_KEY_RESP
{
    unsigned short type;
    unsigned short count;
    unsigned short error;
    unsigned short version;
};


struct READ_KEY_REQ
{
    unsigned short type;
    unsigned short count;
    unsigned short error;
    unsigned short version;
    char           user[ACS_USERID_SIZE];
    unsigned short direction;
    SECKey         key;
};

struct READ_KEY_RESP
{
    unsigned short type;
    unsigned short count;
    unsigned short error;
    unsigned short version;
    SECKey         keys[NUM_SECSERV_KEYS];
};


/*------------------------------------------------------------------------*\
  GET_ACS_INFO: 
\*------------------------------------------------------------------------*/

struct GET_ACS_INFO_REQ
{
    unsigned short type;
    unsigned short count;
    unsigned short error;
    unsigned short version;
};

struct GET_ACS_INFO_RESP
{
    unsigned short type;
    unsigned short count;
    unsigned short error;
    unsigned short version;

    double dataFileTimestampUtc;

    // Some reserved space for future use.
    char unused[100];
};



/*------------------------------------------------------------------------*\
  GET_USER_ACCESS: 
\*------------------------------------------------------------------------*/

struct USER_ACCESS_ITEM
{
    char application[8];
    char event[8];
};


struct GET_USER_ACCESS_REQ
{
    //lint --e{834}  // error 834: (Info -- Operator '-' followed by operator '-' is confusing.  Use parentheses.)
    enum { eMAX_COUNT   = (MAX_NET_MSG_SIZE - sizeof(ACS_MESSAGE_HEADER) - sizeof(char)*ACS_USERID_SIZE) / sizeof(USER_ACCESS_ITEM) };
    enum { eMSG_VERSION = 1 };
    enum { eMSG_TYPE    = GET_USER_ACCESS};

    ACS_MESSAGE_HEADER header;
    char               user[ACS_USERID_SIZE];
    USER_ACCESS_ITEM   items[eMAX_COUNT];
};

struct GET_USER_ACCESS_RESP
{
    enum { eMAX_COUNT   = GET_USER_ACCESS_REQ::eMAX_COUNT };
    enum { eMSG_VERSION = GET_USER_ACCESS_REQ::eMSG_VERSION };
    enum { eMSG_TYPE    = GET_USER_ACCESS_REQ::eMSG_TYPE};

    ACS_MESSAGE_HEADER header;
    char               access[eMAX_COUNT];
};

#pragma pack(pop, __ACS__PACKING__)

/*************************************************************************\
 $History: Acs.h $
 * 
 * *****************  Version 9  *****************
 * User: Derek.oates  Date: 8/06/01    Time: 6:22p
 * Updated in $/CygNet/Source/Support/CygNetCore
 * Cleaned up for Lint.
 * 
 * *****************  Version 8  *****************
 * User: Derek.oates  Date: 7/27/01    Time: 5:37p
 * Updated in $/CygNet/Source/Support/CygNetCore
 * Cleaned up per Lint.
 * 
 * *****************  Version 7  *****************
 * User: Derek.oates  Date: 7/11/01    Time: 4:05p
 * Updated in $/cygnet/Source/Support/CygNetCore
 * Removed unnecessary #ifdef MSC_VER blocks.
 * 
 * *****************  Version 6  *****************
 * User: Derek.oates  Date: 6/29/01    Time: 6:20p
 * Updated in $/CygNet/Source/Support/CygNetCore
 * Added new GET_ACS_INFO message which currently just returns database
 * timestamp.
 * 
 * *****************  Version 5  *****************
 * User: Blf1         Date: 1/26/01    Time: 2:49p
 * Updated in $/CygNet/Source/Support/CygNetCore
 * Fix eMaxCount on GET_USER_ACCESS_REQ.
 * 
 * *****************  Version 4  *****************
 * User: Rhg1         Date: 12/29/00   Time: 5:34p
 * Updated in $/CygNet/Source/Support/CygNetCore
 * defined and implemented wrappers for the GET_USER_ACCESS message
 * 
 * *****************  Version 3  *****************
 * User: Rhg1         Date: 12/11/00   Time: 12:07p
 * Updated in $/CygNet/Source/Support/CygNetCore
 * moved CAcsCache enumeration to global scope
 * 
 * *****************  Version 2  *****************
 * User: Rps1         Date: 10/06/99   Time: 10:12p
 * Updated in $/CygNet/Source/Support/CygNetCore
 * Modified message ids to use common service id base value.
 * 
 * *****************  Version 1  *****************
 * User: Rps1         Date: 9/17/99    Time: 3:56p
 * Created in $/CygNet/Source/Support/CygNetCore
 * New core struct and message files.

\*************************************************************************/

#endif  /*  __ACS_H  */
